import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

enum TagType { primary, success, danger, warning }
enum TagSize { large, medium }

class VantTagTheme {
  final EdgeInsets tagPadding;
  final double tagFontSize;
  final double tagMediumFontSize;
  final double tagLargeFontSize;
  final Color tagTextColor;
  final double tagBorderRadius;
  final double tagRoundBorderRadius;
  final Color tagDangerColor;
  final Color tagPrimaryColor;
  final Color tagSuccessColor;
  final Color tagWarningColor;
  final Color tagDefaultColor;
  final Color tagPlainBackgroundColor;

  double size(TagSize size) {
    switch (size) {
      case TagSize.large:
        return this.tagLargeFontSize;
        break;
      case TagSize.medium:
        return this.tagMediumFontSize;
        break;
      default:
        return this.tagFontSize;
        break;
    }
  }

  Color color(TagType type) {
    switch (type) {
      case TagType.danger:
        return this.tagDangerColor;
        break;
      case TagType.primary:
        return this.tagPrimaryColor;
        break;
      case TagType.warning:
        return this.tagWarningColor;
        break;
      case TagType.success:
        return this.tagSuccessColor;
      default:
        return this.tagDefaultColor;
    }
  }

  const VantTagTheme({
    this.tagPadding ,
    this.tagFontSize,
    this.tagMediumFontSize,
    this.tagLargeFontSize,
    this.tagTextColor,
    this.tagBorderRadius,
    this.tagRoundBorderRadius,
    this.tagDangerColor,
    this.tagPrimaryColor,
    this.tagSuccessColor,
    this.tagWarningColor,
    this.tagDefaultColor,
    this.tagPlainBackgroundColor,
  });

  VantTagTheme copyWith({
    EdgeInsets tagPadding,
    double tagFontSize,
    double tagMediumFontSize,
    double tagLargeFontSize,
    Color tagTextColor,
    double tagBorderRadius,
    double tagRoundBorderRadius,
    Color tagDangerColor,
    Color tagPrimaryColor,
    Color tagSuccessColor,
    Color tagWarningColor,
    Color tagDefaultColor,
    Color tagPlainBackgroundColor,
  }) {
    if ((tagPadding == null || identical(tagPadding, this.tagPadding)) &&
        (tagFontSize == null || identical(tagFontSize, this.tagFontSize)) &&
        (tagMediumFontSize == null ||
            identical(tagMediumFontSize, this.tagMediumFontSize)) &&
        (tagLargeFontSize == null ||
            identical(tagLargeFontSize, this.tagLargeFontSize)) &&
        (tagTextColor == null || identical(tagTextColor, this.tagTextColor)) &&
        (tagBorderRadius == null ||
            identical(tagBorderRadius, this.tagBorderRadius)) &&
        (tagRoundBorderRadius == null ||
            identical(tagRoundBorderRadius, this.tagRoundBorderRadius)) &&
        (tagDangerColor == null ||
            identical(tagDangerColor, this.tagDangerColor)) &&
        (tagPrimaryColor == null ||
            identical(tagPrimaryColor, this.tagPrimaryColor)) &&
        (tagSuccessColor == null ||
            identical(tagSuccessColor, this.tagSuccessColor)) &&
        (tagWarningColor == null ||
            identical(tagWarningColor, this.tagWarningColor)) &&
        (tagDefaultColor == null ||
            identical(tagDefaultColor, this.tagDefaultColor)) &&
        (tagPlainBackgroundColor == null ||
            identical(tagPlainBackgroundColor, this.tagPlainBackgroundColor))) {
      return this;
    }

    return new VantTagTheme(
      tagPadding: tagPadding ?? this.tagPadding,
      tagFontSize: tagFontSize ?? this.tagFontSize,
      tagMediumFontSize: tagMediumFontSize ?? this.tagMediumFontSize,
      tagLargeFontSize: tagLargeFontSize ?? this.tagLargeFontSize,
      tagTextColor: tagTextColor ?? this.tagTextColor,
      tagBorderRadius: tagBorderRadius ?? this.tagBorderRadius,
      tagRoundBorderRadius: tagRoundBorderRadius ?? this.tagRoundBorderRadius,
      tagDangerColor: tagDangerColor ?? this.tagDangerColor,
      tagPrimaryColor: tagPrimaryColor ?? this.tagPrimaryColor,
      tagSuccessColor: tagSuccessColor ?? this.tagSuccessColor,
      tagWarningColor: tagWarningColor ?? this.tagWarningColor,
      tagDefaultColor: tagDefaultColor ?? this.tagDefaultColor,
      tagPlainBackgroundColor:
          tagPlainBackgroundColor ?? this.tagPlainBackgroundColor,
    );
  }

  @override
  String toString() {
    return 'VantTag{tagPadding: $tagPadding, tagFontSize: $tagFontSize, tagMediumFontSize: $tagMediumFontSize, tagLargeFontSize: $tagLargeFontSize, tagTextColor: $tagTextColor, tagBorderRadius: $tagBorderRadius, tagRoundBorderRadius: $tagRoundBorderRadius, tagDangerColor: $tagDangerColor, tagPrimaryColor: $tagPrimaryColor, tagSuccessColor: $tagSuccessColor, tagWarningColor: $tagWarningColor, tagDefaultColor: $tagDefaultColor, tagPlainBackgroundColor: $tagPlainBackgroundColor}';
  }
}
