import 'package:flutter/material.dart';
import 'package:vant/src/theme/card.dart';
import 'package:vant/src/theme/color.dart';
import 'package:vant/src/theme/empty.dart';
import 'package:vant/src/theme/field.dart';
import 'package:vant/src/theme/font.dart';
import 'package:vant/src/theme/notify.dart';
import 'package:vant/src/theme/padding.dart';
import 'package:vant/src/theme/pagination.dart';
import 'package:vant/src/theme/search.dart';
import 'package:vant/src/theme/tag.dart';

import 'basic.dart';
import 'button.dart';
import 'cell.dart';
import 'cell_group.dart';

class VantTheme extends InheritedWidget {
  final VantThemeData data;

  VantTheme({Key key, @required this.data, @required Widget child})
      : super(key: key, child: child);

  @override
  bool updateShouldNotify(VantTheme old) {
    return data != old.data;
  }

  /// Use this method to get the current [VantThemeData] instance
  static VantThemeData of(BuildContext context) {
    final streamChatTheme =
        context.dependOnInheritedWidgetOfExactType<VantTheme>();

    if (streamChatTheme == null) {
      throw Exception('无VantTheme组件');
    }

    return streamChatTheme.data;
  }
}

class VantThemeData {
  final VantCellTheme cell;
  final VantCardTheme card;
  final VantBasicTheme basic;
  final VantButtonTheme button;
  final VantCellGroupTheme cellGroup;
  final VantColorTheme color;
  final VantEmptyTheme empty;
  final VantFieldTheme field;
  final VantFontTheme font;
  final VantNotifyTheme notify;
  final VantPaddingTheme padding;
  final VantPaginationTheme pagination;
  final VantSearchTheme search;
  final VantTagTheme tag;

  factory VantThemeData.fromTheme(ThemeData theme) {
    final defaultTheme = getDefaultTheme(theme);
    return defaultTheme.copyWith();
  }

  static VantThemeData getDefaultTheme(ThemeData theme) {
    final VantColorTheme color = VantColorTheme(
      red: Colors.red,
      blue: Colors.blue,
      black: Colors.black,
      white: Colors.white,
      green: Colors.green,
      orange: Colors.orange,
      gray1: Color(0xfff7f8fa),
      gray2: Color(0xfff2f3f5),
      gray3: Color(0xffebedf0),
      gray4: Color(0xffdcdee0),
      gray5: Color(0xffc8c9cc),
      gray6: Color(0xff969799),
      gray7: Color(0xff646566),
      gray8: Color(0xff323233),
      orangeDark: Color(0xffed6a0c),
      orangeLight: Color(0xfffffbe8),
      activeColor: Color(0xfff7f8fa),
      activeOpacity: 0.7,
      gradientRed:
          LinearGradient(colors: [Color(0xffff6034), Color(0xffee0a24)]),
      gradientOrange:
          LinearGradient(colors: [Color(0xffffd01e), Color(0xffff8917)]),
      disabledOpacity: 0.5,
      backgroundColorLight: Color(0xfffafafa),
      backgroundColor: Color(0xfff7f8fa),
      textColor: Color(0xff323233),
    );

    final VantBasicTheme basic = VantBasicTheme(
      borderColor: color.gray3,
      borderRadiusMax: 999.0,
      borderWidthBase: 1.0,
      borderRadiusSm: 2.0,
      borderRadiusMd: 4.0,
      borderRadiusLg: 8.0,
    );

    final VantPaddingTheme padding = VantPaddingTheme(
      paddingBase: 4.0,
      paddingXs: 4.0 * 2,
      paddingSm: 4.0 * 3,
      paddingMd: 4.0 * 4,
      paddingLg: 4.0 * 6,
      paddingXl: 4.0 * 8,
    );

    final VantFontTheme font = VantFontTheme(
      fontSizeMd: 14.0,
      fontSizeSm: 12.0,
      fontWeightBold: 500,
      fontSizeLg: 16.0,
      fontSizeXs: 10.0,
    );

    final VantCardTheme card = VantCardTheme(
      cardPadding: EdgeInsets.symmetric(
          vertical: padding.paddingXs, horizontal: padding.paddingMd),
      cardFontSize: font.fontSizeSm,
      cardTextColor: color.textColor,
      cardBackgroundColor: color.backgroundColorLight,
      cardThumbSize: 88.0,
      cardThumbBorderRadius: basic.borderRadiusLg,
      cardTitleLineHeight: 16.0,
      cardDescColor: color.gray7,
      cardDescLineHeight: 20.0,
      cardPriceColor: color.gray8,
      cardOriginPriceColor: color.gray6,
      cardNumColor: color.gray6,
      cardPriceFontSize: font.fontSizeSm,
      cardOriginPriceFontSize: font.fontSizeXs,
      cardPriceIntegerFontSize: font.fontSizeLg,
    );

    final VantCellTheme cell = VantCellTheme(
      cellBorderColor: basic.borderColor,
      cellRequiredColor: color.red,
      cellActiveColor: color.activeColor,
      cellBackgroundColor: color.white,
      cellLabelMarginTop: padding.paddingBase,
      cellFontSize: font.fontSizeMd,
      cellLabelColor: color.gray6,
      cellLabelFontSize: font.fontSizeSm,
      cellLabelLineHeight: 18.0,
      cellIconSize: 16.0,
      cellHorizontalPadding: padding.paddingSm,
      cellLargeLabelFontSize: font.fontSizeMd,
      cellLargeTitleFontSize: font.fontSizeLg,
      cellLargeVerticalPadding: padding.paddingMd,
      cellLineHeight: 24.0,
      cellTextColor: color.textColor,
      cellRightIconColor: color.gray6,
      cellVerticalPadding: padding.paddingSm,
      cellValueColor: color.gray6,
    );

    final VantTagTheme tag = VantTagTheme(
      tagDangerColor: color.red,
      tagPrimaryColor: color.blue,
      tagSuccessColor: color.green,
      tagTextColor: color.white,
      tagFontSize: font.fontSizeXs,
      tagWarningColor: color.orange,
      tagDefaultColor: color.gray6,
      tagLargeFontSize: font.fontSizeMd,
      tagMediumFontSize: font.fontSizeSm,
      tagPlainBackgroundColor: color.white,
      tagBorderRadius: basic.borderRadiusSm,
      tagRoundBorderRadius: basic.borderRadiusMax,
      tagPadding: EdgeInsets.symmetric(vertical: 3.0, horizontal: padding.paddingBase),
    );

    return VantThemeData(
      tag: tag,
      cell: cell,
      font: font,
      card: card,
      color: color,
      basic: basic,
      padding: padding,
      empty: VantEmptyTheme(
        emptyPadding:
            EdgeInsets.symmetric(vertical: padding.paddingXl, horizontal: 0),
        emptyImageSize: 160,
        emptyDescriptionMarginTop: padding.paddingMd,
        emptyDescriptionPadding:
            EdgeInsets.symmetric(vertical: 0, horizontal: 60.0),
        emptyDescriptionColor: color.gray6,
        emptyDescriptionFontSize: 14.0,
        emptyBottomMarginTop: 24.0,
        emptyDescriptionLineHeight: 20.0,
      ),
      field: VantFieldTheme(
        fieldLabelWidth: 80.0,
        fieldLabelColor: color.gray7,
        fieldLabelMarginRight: padding.paddingSm,
        fieldDisabledTextColor: color.gray5,
        fieldClearIconColor: color.gray5,
        fieldClearIconSize: 16,
        fieldErrorMessageColor: color.red,
        fieldErrorMessageTextSize: 12.0,
        fieldIconSize: 16.0,
        fieldInputDisabledTextColor: color.gray5,
        fieldInputTextColor: color.textColor,
        fieldInputErrorTextColor: color.red,
        fieldPlaceholderTextColor: color.gray5,
        fieldRightIconColor: color.gray6,
        fieldTextAreaMinHeight: 60.0,
        fieldWordLimitColor: color.gray7,
        fieldWordLimitFontSize: font.fontSizeSm,
        fieldWordLimitLineHeight: 16.0,
        fieldRequiredColor: color.red,
      ),
      button: VantButtonTheme(
        buttonRoundBorderRadius: basic.borderRadiusMax,
        buttonBorderRadius: basic.borderRadiusSm,
        buttonHairBorderWidth: 0.5,
        buttonBorderWidth: basic.borderWidthBase,
        buttonDangerBackgroundColor: color.red,
        buttonDangerBorderColor: color.red,
        buttonPrimaryBackgroundColor: color.green,
        buttonMiniFontSize: font.fontSizeXs,
        buttonLargeHeight: 50.0,
        buttonInfoBorderColor: color.blue,
        buttonDisabledOpacity: color.disabledOpacity,
        buttonDefaultFontSize: font.fontSizeMd,
        buttonDefaultColor: color.textColor,
        buttonDefaultBorderColor: basic.borderColor,
        buttonDefaultHeight: 44.0,
        buttonDangerColor: color.white,
        buttonDefaultLineHeight: 1.2,
        buttonInfoBackgroundColor: color.blue,
        buttonDefaultBackgroundColor: color.white,
        buttonInfoColor: color.white,
        buttonNormalFontSize: font.fontSizeMd,
        buttonMiniHeight: 22.0,
        buttonPrimaryBorderColor: color.green,
        buttonPrimaryColor: color.white,
        buttonPlainBackgroundColor: color.white,
        buttonSmallFontSize: font.fontSizeSm,
        buttonSmallHeight: 30.0,
        buttonWarningBackgroundColor: color.orange,
        buttonWarningBorderColor: color.orange,
        buttonWarningColor: color.white,
      ),
      notify: VantNotifyTheme(
        notifyDangerBackgroundColor: null,
        notifyFontSize: null,
        notifyLineHeight: null,
        notifyPadding: null,
        notifyPrimaryBackgroundColor: null,
        notifySuccessBackgroundColor: null,
        notifyTextColor: null,
        notifyWarningBackgroundColor: null,
      ),
      search: VantSearchTheme(
        searchActionFontSize: null,
        searchActionPadding: null,
        searchActionTextColor: null,
        searchBackgroundColor: null,
        searchContentBackgroundColor: null,
        searchInputHeight: null,
        searchLabelColor: null,
        searchLabelFontSize: null,
        searchLabelPadding: null,
        searchLeftIconColor: null,
        searchPadding: null,
      ),
      cellGroup: VantCellGroupTheme(
        cellGroupBackgroundColor: color.white,
        cellGroupTitleColor: color.gray6,
        cellGroupTitleFontSize: font.fontSizeMd,
        cellGroupTitleLineHeight: 16.0,
        cellGroupTitlePadding: EdgeInsets.only(
          top: padding.paddingMd,
          right: padding.paddingMd,
          bottom: padding.paddingXs,
          left: padding.paddingXs,
        ),
      ),
      pagination: VantPaginationTheme(
        paginationBackgroundColor: null,
        paginationDescColor: null,
        paginationFontSize: null,
        paginationDisabledOpacity: null,
        paginationHeight: null,
        paginationItemDefaultColor: null,
        paginationItemDisabledBackgroundColor: null,
        paginationItemDisabledColor: null,
        paginationItemWidth: null,
      ),
    );
  }

  const VantThemeData({
    this.tag,
    this.cell,
    this.card,
    this.basic,
    this.button,
    this.color,
    this.empty,
    this.field,
    this.font,
    this.notify,
    this.search,
    this.padding,
    this.cellGroup,
    this.pagination,
  });

  VantThemeData copyWith({
    VantCellTheme cell,
    VantCardTheme card,
    VantBasicTheme basic,
    VantButtonTheme button,
    VantCellGroupTheme cellGroup,
    VantColorTheme color,
    VantEmptyTheme empty,
    VantFieldTheme field,
    VantFontTheme font,
    VantNotifyTheme notify,
    VantPaddingTheme padding,
    VantPaginationTheme pagination,
    VantSearchTheme search,
    VantTagTheme tag,
  }) {
    if ((cell == null || identical(cell, this.cell)) &&
        (card == null || identical(card, this.card)) &&
        (basic == null || identical(basic, this.basic)) &&
        (button == null || identical(button, this.button)) &&
        (cellGroup == null || identical(cellGroup, this.cellGroup)) &&
        (color == null || identical(color, this.color)) &&
        (empty == null || identical(empty, this.empty)) &&
        (field == null || identical(field, this.field)) &&
        (font == null || identical(font, this.font)) &&
        (notify == null || identical(notify, this.notify)) &&
        (padding == null || identical(padding, this.padding)) &&
        (pagination == null || identical(pagination, this.pagination)) &&
        (search == null || identical(search, this.search)) &&
        (tag == null || identical(tag, this.tag))) {
      return this;
    }

    return new VantThemeData(
      cell: cell ?? this.cell,
      card: card ?? this.card,
      basic: basic ?? this.basic,
      button: button ?? this.button,
      cellGroup: cellGroup ?? this.cellGroup,
      color: color ?? this.color,
      empty: empty ?? this.empty,
      field: field ?? this.field,
      font: font ?? this.font,
      notify: notify ?? this.notify,
      padding: padding ?? this.padding,
      pagination: pagination ?? this.pagination,
      search: search ?? this.search,
      tag: tag ?? this.tag,
    );
  }
}
