// +----------------------------------------------------------------------
// | 按钮组件
// | https://youzan.github.io/vant/#/zh-CN/button
// +----------------------------------------------------------------------

import 'package:flutter/material.dart';
import 'package:vant/vant.dart';


class VanButton extends StatefulWidget {
  // 是否为朴素按钮
  final bool plain;

  // 是否为块级元素
  final bool block;

  // 是否为圆形按钮
  final bool round;

  // 按钮背景颜色
  final Color color;

  // 按钮图标
  final Widget icon;

  // 是否为方形按钮
  final bool square;

  // 按钮文字
  final String label;

  // 是否为加载中
  final bool loading;

  // 是否禁用
  final bool disabled;

  // 按钮高度
  final double height;

  // 是否使用 0.5px 边框
  final bool hairline;

  // 按钮类型
  final ButtonType type;

  // 按钮尺寸
  final ButtonSize size;

  // 按钮文字颜色
  final Color labelColor;

  // 点击
  final VoidCallback onPressed;

  // 长按
  final VoidCallback onLongPress;

  VanButton({
    Key key,
    this.type,
    this.size,
    this.icon,
    this.color,
    this.height,
    this.plain = false,
    this.round = false,
    this.block = false,
    this.square = false,
    this.loading = false,
    this.hairline = false,
    this.disabled = false,
    this.labelColor,
    this.onLongPress,
    this.label,
    this.onPressed,
  }) : super(key: key);

  @override
  _VanButtonState createState() => _VanButtonState();
}

class _VanButtonState extends State<VanButton> {
  @override
  Widget build(BuildContext context) {
    final VantButtonTheme buttonTheme = VantTheme.of(context).button;
    final Color backgroundColor = this.widget.plain
        ?  buttonTheme.buttonPlainBackgroundColor
        : this.widget.color ?? buttonTheme.backgroundColor(this.widget.type);

    final BorderRadius borderRadius = this.widget.square
        ? BorderRadius.zero
        : BorderRadius.circular(
            this.widget.round
                ? buttonTheme.buttonRoundBorderRadius
                : buttonTheme.buttonBorderRadius,
          );

    return Opacity(
      opacity: 1,
      child: DecoratedBox(
        decoration: BoxDecoration(
          color: backgroundColor,
          borderRadius: borderRadius,
          border: Border.all(
            color: this.widget.color ?? buttonTheme.borderColor(this.widget.type),
            width: this.widget.hairline
                ? buttonTheme.buttonHairBorderWidth
                : buttonTheme.buttonBorderWidth,
          ),
        ),
        child: Material(
          type: MaterialType.transparency,
          child: InkWell(
            onTap: this.widget.onPressed,
            onLongPress: this.widget.onLongPress,
            borderRadius: borderRadius,
            child: Container(
              height: buttonTheme.height(this.widget.size),
              padding: buttonTheme.padding(this.widget.size),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                mainAxisSize:
                    this.widget.block ? MainAxisSize.max : MainAxisSize.min,
                children: <Widget>[
                  if (this.widget.loading) this._loading(),
                  if (this.widget.icon != null) this._icon(),
                  if (this.widget.label != null) this._label(),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  Widget _label() {
    final VantButtonTheme buttonTheme = VantTheme.of(context).button;
    final textColor = this.widget.labelColor ??
        (this.widget.plain
            ? this.widget.color ?? buttonTheme.backgroundColor(this.widget.type)
            : buttonTheme.textColor(this.widget.type));

    return Text(
      this.widget.label,
      style: TextStyle(
        fontSize: buttonTheme.fontSize(this.widget.size),
        color: textColor,
      ),
    );
  }

  Widget _icon() {
    return Padding(
      child: this.widget.icon,
      padding: this.widget.label == null
          ? EdgeInsets.zero
          : EdgeInsets.only(right: 5),
    );
  }

  Widget _loading() {
    final buttonTheme = VantTheme.of(context).button;

    return Padding(
      padding: EdgeInsets.only(right: 5),
      child: SizedBox(
        width: buttonTheme.fontSize(this.widget.size),
        height: buttonTheme.fontSize(this.widget.size),
        child: CircularProgressIndicator(
          valueColor: AlwaysStoppedAnimation(
            buttonTheme.textColor(this.widget.type),
          ),
          backgroundColor: buttonTheme.backgroundColor(this.widget.type),
          strokeWidth: VantTheme.of(context).basic.borderWidthBase,
        ),
      ),
    );
  }
}
