// +----------------------------------------------------------------------
// | 单元格组件
// | https://youzan.github.io/vant/#/zh-CN/button
// +----------------------------------------------------------------------

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:vant/vant.dart';
import 'dart:io';

enum CellSize { large }

class VanCell extends StatefulWidget {
  // 左侧标题
  final String title;

  // 	右侧内容
  final String value;

  // 	标题下方的描述信息
  final String label;

  // 左侧图标
  final IconData icon;

  // 是否显示表单必填星号
  final bool require;

  // 自定义右侧图标
  final Widget rightIcon;

  // 自定义单元格最右侧的额外内容
  final Widget extra;

  // 是否使内容垂直居中
  final bool center;

  final bool clickable;

  final bool isLink;

  // 	单元格大小
  final CellSize size;

  // 自定义左侧 title 的内容
  final Widget widgetTitle;

  // 自定义左侧图标
  final Widget widgetValue;

  // 自定义标题下方 label 的内容
  final Widget widgetLabel;

  // 点击
  final VoidCallback onPressed;

  // 长按
  final VoidCallback onLongPress;

  VanCell({
    Key key,
    this.size,
    this.icon,
    this.title,
    this.label,
    this.value,
    this.extra,
    this.isLink = false,
    this.center,
    this.require = false,
    this.clickable = false,
    this.onPressed,
    this.rightIcon,
    this.widgetValue,
    this.widgetTitle,
    this.widgetLabel,
    this.onLongPress,
  }) : super(key: key);



  @override
  _VanCellState createState() => _VanCellState();
}

class _VanCellState extends State<VanCell> {
  @override
  Widget build(BuildContext context) {
    return InkWell(
      child: this._builderContainer(),
      onTap: this.widget.onPressed,
      onLongPress: this.widget.onLongPress,
    );
  }

  Widget _builderContainer() {
    final cellTheme = VantTheme.of(context).cell;

    return DecoratedBox(
      decoration: BoxDecoration(color: cellTheme.cellBackgroundColor),
      child: Material(
        child: InkWell(
          highlightColor: (this.widget.clickable || this.widget.isLink)
              ? Theme.of(context).highlightColor
              : Colors.transparent,
          splashColor: (this.widget.clickable || this.widget.isLink)
              ? Theme.of(context).splashColor
              : Colors.transparent,
          child: Container(
            child: this._builderCell(),
            padding: EdgeInsets.symmetric(
              vertical: this.widget.size == CellSize.large
                  ? cellTheme.cellLargeVerticalPadding
                  : cellTheme.cellVerticalPadding,
              horizontal: cellTheme.cellHorizontalPadding,
            ),
          ),
          onTap: this.widget.onPressed,
          onLongPress: this.widget.onLongPress,
        ),
        type: MaterialType.transparency,
      ),
    );
  }

  Widget _builderCell() {
    final cellTheme = VantTheme.of(context).cell;

    return Flex(
      direction: Axis.horizontal,
      mainAxisSize: MainAxisSize.max,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        this._buildLeft(),

        if (this.widget.title != null || this.widget.widgetTitle != null)
        Expanded(
            child: Container(
                child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[

              this.widget.widgetTitle != null
                  ? this.widget.widgetTitle
                  : Text(
                this.widget.title,
                style: TextStyle(
                  fontSize: this.widget.size == CellSize.large
                      ? cellTheme.cellLargeTitleFontSize
                      : cellTheme.cellFontSize,
                ),
              ),

            if (this.widget.label != null || this.widget.widgetLabel != null)
              this.widget.widgetLabel != null
                  ? this.widget.widgetLabel
                  : Container(
                padding: EdgeInsets.only(top: 3),
                child: Text(
                  this.widget.label,
                  style: TextStyle(
                    color: cellTheme.cellLabelColor,
                    fontSize: this.widget.size == CellSize.large
                        ? cellTheme.cellLargeLabelFontSize
                        : cellTheme.cellLabelFontSize,
                  ),
                ),
              ),
          ],
        ))),
        this._buildRight(),
      ],
    );
  }

  Widget _buildLeft() {
    final cellTheme = VantTheme.of(context).cell;

    return Row(
      children: <Widget>[
        if (this.widget.require)
          Text(
            "*",
            style: TextStyle(
              fontSize: cellTheme.cellFontSize,
              color: cellTheme.cellRequiredColor,
            ),
          ),
        if (this.widget.icon != null)
          Icon(
            this.widget.icon,
            size: cellTheme.cellIconSize,
            color: cellTheme.cellTextColor,
          ),
      ],
    );
  }

  Widget _buildRight() {
    final cellTheme = VantTheme.of(context).cell;

    return Expanded(
      flex: this.widget.title == null ? 1 : 0,
      child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            if (this.widget.value != null || this.widget.widgetValue != null)
              if (this.widget.widgetValue != null)
                this.widget.widgetValue
            else
              Text(
                this.widget.value,
                style: TextStyle(
                  fontSize: cellTheme.cellFontSize,
                  color: this.widget.title != null ||
                          this.widget.widgetTitle != null
                      ? cellTheme.cellValueColor
                      : cellTheme.cellTextColor,
                ),
              ),
            if (this.widget.isLink) this._builderRightIcon()
          ]),
    );
  }

  Widget _builderRightIcon() {
    if (Platform.isIOS)
      return Icon(CupertinoIcons.right_chevron);
    else
      return Icon(Icons.chevron_right);
  }
}
