// +----------------------------------------------------------------------
// | 单元格分组组件
// | https://youzan.github.io/vant/#/zh-CN/button
// +----------------------------------------------------------------------

import 'package:flutter/material.dart';
import 'package:vant/src/theme/theme.dart';

class VanCellGroup extends StatefulWidget {
  // 分组标题
  final String title;

  // 是否显示外边框
  final bool border;

  // 默认插槽
  final List<Widget> children;

  final EdgeInsets padding;

  VanCellGroup({
    Key key,
    this.title,
    this.border = true,
    this.padding,
    @required this.children,
  }) : super(key: key);

  @override
  _VanCellGroupState createState() => _VanCellGroupState();
}

class _VanCellGroupState extends State<VanCellGroup> {
  @override
  Widget build(BuildContext context) {
    return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          if (this.widget.title != null) this._title(),
          if (this.widget.children.length > 0)
            Container(
              padding: this.widget.padding,
              child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: this._list()),
              decoration: this._builderBorder(),
            )
        ]);
  }

  _builderBorder() {
     if(this.widget.border)
       return BoxDecoration(
      border: Border(
        top: BorderSide(
          color: VantTheme.of(context).cell.cellBorderColor,
          width: VantTheme.of(context).basic.borderWidthBase,
        ),
        bottom: BorderSide(
          color: VantTheme.of(context).cell.cellBorderColor,
          width: VantTheme.of(context).basic.borderWidthBase,
        ),
      ),
    );
  }

  Widget _title() {
    return Padding(
      padding: VantTheme.of(context).cellGroup.cellGroupTitlePadding,
      child: Text(
        this.widget.title,
        style: TextStyle(
          fontSize: VantTheme.of(context).cellGroup.cellGroupTitleFontSize,
          color: Colors.grey,
        ),
      ),
    );
  }

  List<Widget> _list() {
    final int length = this.widget.border
        ? this.widget.children.length * 2 - 1
        : this.widget.children.length;

    return List.generate(length, (index) {
      if (index.isOdd && this.widget.border) {
        return Container(
          color: Colors.white,
          child: Divider(
            height: 0.5,
            indent: 14.0,
            endIndent: 14.0,
          ),
        );
      }

      index = this.widget.border ? index ~/ 2 : index;
      return this.widget.children[index];
    });
  }
}
