// +----------------------------------------------------------------------
// | 输入框组件
// | https://youzan.github.io/vant/#/zh-CN/button
// +----------------------------------------------------------------------

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:vant/vant.dart';

enum FieldSize { large, small, mini }
enum LabelAlign { center, left }
enum InputAlign { center, left }
enum ErrorMessageAlign { center, left }

class VanField extends StatefulWidget {
  // 输入框左侧文本
  final String label;

  // 大小
  final FieldSize size;

  // 输入框类型
  final TextInputType keyboardType;

  // 键盘右下角的类型
  final TextInputAction textInputAction;

  // 输入控制
  final TextEditingController controller;

  // 是否只读
  final bool readonly;

  // 是否显示表单必填星号
  final bool require;

  // 输入的最大字符数
  final int maxLength;

  // 占位提示文字
  final String placeholder;

  // 是否禁用输入框
  final bool disabled;

  // 是否启用清除控件
  final bool clearable;

  // 自动聚焦
  final bool autofocus;

  // 是否开启点击反馈
  final bool clickable;

  // 是否展示右侧箭头并开启点击反馈
  final bool isLink;

  // 是否将输入内容标红
  final bool error;

  // 底部错误提示文案，为空时不展示
  final String errorMessage;

  // 左侧文本宽度
  final double labelWidth;

  // 左侧文本对齐方式
  final TextAlign labelAlign;

  // 输入框对齐方式
  final TextAlign inputAlign;

  // 错误提示文案对齐方式
  final ErrorMessageAlign errorMessageAlign;

  // 是否自适应内容高度
  final bool autoSize;

  // 左侧图标
  final IconData leftIcon;

  // 右侧图标
  final Widget rightIcon;

  // 焦点控制
  final FocusNode focusNode;

  VanField({
    Key key,
    this.size,
    this.keyboardType,
    this.label,
    this.error = false,
    this.leftIcon,
    this.autoSize,
    this.maxLength,
    this.isLink = false,
    this.require = false,
    this.disabled = false,
    this.readonly = false,
    this.autofocus = false,
    this.clearable = false,
    this.clickable = false,
    this.rightIcon,
    this.labelWidth,
    this.inputAlign = TextAlign.start,
    this.labelAlign = TextAlign.start,
    this.placeholder,
    this.errorMessage,
    this.textInputAction = TextInputAction.done,
    @required this.controller,
    this.errorMessageAlign,
    this.focusNode,
  }) : super(key: key);

  @override
  _VanFieldState createState() => _VanFieldState();
}

class _VanFieldState extends State<VanField> {
  @override
  Widget build(BuildContext context) {
    return InkWell(
      child: this._builderContainer(),
//      onTap: this.widget.onPressed,
//      onLongPress: this.widget.onLongPress,
    );
  }

  Widget _builderContainer() {
    final cellTheme = VantTheme.of(context).cell;

    return DecoratedBox(
      decoration: BoxDecoration(color: cellTheme.cellBackgroundColor),
      child: Material(
        child: InkWell(
          highlightColor: (this.widget.clickable || this.widget.isLink)
              ? Theme.of(context).highlightColor
              : Colors.transparent,
          splashColor: (this.widget.clickable || this.widget.isLink)
              ? Theme.of(context).splashColor
              : Colors.transparent,
          child: Container(
            child: this._builderCell(),
            padding: EdgeInsets.symmetric(
              vertical: cellTheme.cellVerticalPadding,
              horizontal: cellTheme.cellHorizontalPadding,
            ),
          ),
        ),
        type: MaterialType.transparency,
      ),
    );
  }

  Widget _builderCell() {
    final fieldTheme = VantTheme.of(context).field;

    return Container(child: Flex(
      direction: Axis.horizontal,
      mainAxisSize: MainAxisSize.max,
      crossAxisAlignment: CrossAxisAlignment.center,
//      mainAxisAlignment: MainAxisAlignment.center,
      children: <Widget>[
        this._buildLeft(),
        Flexible(
          fit: FlexFit.tight,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              this._input(),
              if (this.widget.error)
                Container(
                  padding: EdgeInsets.only(top: 3),
                  child: Text(
                    "错误信息",
                    style: TextStyle(
                      fontSize: fieldTheme.fieldErrorMessageTextSize,
                      color: fieldTheme.fieldErrorMessageColor,
                    ),
                  ),
                )
            ],
          ),
        ),
        this._buildRight(),
      ],
    ));
  }

  Widget _buildLeft() {
    final fieldTheme = VantTheme.of(context).field;

    return Row(children: <Widget>[
      if (this.widget.require)
        Text(
          "*",
          style: TextStyle(
            fontSize: 14.0,
            color: fieldTheme.fieldRequiredColor,
          ),
        ),
      if (this.widget.leftIcon != null)
        Icon(
          this.widget.leftIcon,
          size: fieldTheme.fieldIconSize,
        ),
      if (this.widget.label != null)
        SizedBox(
            width: this.widget.label != null ? fieldTheme.fieldLabelWidth : 0,
            child: Text(
              this.widget.label,
              textAlign: this.widget.labelAlign,
              style: TextStyle(fontSize: 14.0),
            ),
        )
    ]);
  }

  Widget _buildRight() {
    return Container(
      child: Row(
        children: <Widget>[Text("aaa")],
      ),
    );
  }

  Widget _input(){
    final fieldTheme = VantTheme.of(context).field;

    return Container(
        child: TextFormField(
      controller: this.widget.controller,
      focusNode: widget.focusNode,
      keyboardType: widget.keyboardType,
      textInputAction: widget.textInputAction,
      textAlign: widget.inputAlign,
      autofocus: widget.autofocus,
      readOnly: widget.readonly,
      enabled: !widget.disabled,
      style: TextStyle(),
      decoration: InputDecoration(
        isDense: true,
        contentPadding: EdgeInsets.symmetric(vertical: 5),
        hintText: widget.placeholder,
        hintStyle: TextStyle(
          color: widget.error
              ? fieldTheme.fieldInputErrorTextColor
              : fieldTheme.fieldPlaceholderTextColor,
          fontSize: 14.0,
        ),
        // counterText: widget.type != "textarea" ? "" : null,
        border: InputBorder.none,
        errorText: widget.errorMessage,
        errorStyle: TextStyle(
          fontSize: fieldTheme.fieldErrorMessageTextSize,
          color: fieldTheme.fieldRequiredColor,
        ),
      ),
    ));
  }
}
