// +----------------------------------------------------------------------
// | 输入框组件
// | https://youzan.github.io/vant/#/zh-CN/button
// +----------------------------------------------------------------------

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:vant/vant.dart';

enum FieldSize { large }

class VanField extends StatefulWidget {
  // 输入框左侧文本
  final String label;

  // 大小
  final FieldSize size;

  // 输入框类型
  final TextInputType keyboardType;

  // 键盘右下角的类型
  final TextInputAction textInputAction;

  // 输入控制
  final TextEditingController controller;

  // 是否只读
  final bool readonly;

  // 是否显示表单必填星号
  final bool require;

  // 输入的最大字符数
  final int maxLength;

  // 占位提示文字
  final String placeholder;

  // 是否禁用输入框
  final bool disabled;

  // 是否启用清除控件
  final bool clearable;

  // 自动聚焦
  final bool autofocus;

  // 是否开启点击反馈
  final bool clickable;

  // 是否展示右侧箭头并开启点击反馈
  final bool isLink;

  // 是否将输入内容标红
  final bool error;

  // 底部错误提示文案，为空时不展示
  final String errorMessage;

  // 左侧文本宽度
  final double labelWidth;

  // 左侧文本对齐方式
  final TextAlign labelAlign;

  // 输入框对齐方式
  final TextAlign inputAlign;

  // 错误提示文案对齐方式
  final Alignment errorMessageAlign;

  // 是否自适应内容高度
  final bool autoSize;

  final int rows;

  // 左侧图标
  final IconData leftIcon;

  // 右侧图标
  final IconData rightIcon;
  final Widget rightWidget;

  final bool showWordLimit;

  VanField({
    Key key,
    this.size,
    this.keyboardType,
    this.label,
    this.error = false,
    this.leftIcon,
    this.autoSize = false,
    this.maxLength,
    this.isLink = false,
    this.require = false,
    this.rows = 1,
    this.disabled = false,
    this.readonly = false,
    this.autofocus = false,
    this.clearable = false,
    this.clickable = false,
    this.showWordLimit = false,
    this.rightIcon,
    this.labelWidth,
    this.inputAlign = TextAlign.start,
    this.labelAlign = TextAlign.start,
    this.rightWidget,
    this.placeholder,
    this.errorMessage,
    this.textInputAction = TextInputAction.done,
    @required this.controller,
    this.errorMessageAlign,
  }) : super(key: key);

  @override
  _VanFieldState createState() => _VanFieldState();
}

class _VanFieldState extends State<VanField> {
  FocusNode _focusNode = FocusNode();
  bool showClearable = false;

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return InkWell(
      child: Container(child: this._builderContainer(), color: Colors.red),
//      onTap: this.widget.onPressed,
//      onLongPress: this.widget.onLongPress,
    );
  }

  Widget _builderContainer() {
    final cellTheme = VantTheme.of(context).cell;

    return DecoratedBox(
      decoration: BoxDecoration(color: cellTheme.cellBackgroundColor),
      child: Material(
        child: InkWell(
          highlightColor: (this.widget.clickable || this.widget.isLink)
              ? Theme.of(context).highlightColor
              : Colors.transparent,
          splashColor: (this.widget.clickable || this.widget.isLink)
              ? Theme.of(context).splashColor
              : Colors.transparent,
          child: Container(
            child: this._builderCell(),
            padding: EdgeInsets.symmetric(
              vertical: this.widget.rightWidget != null
                  ? 6
                  : this.widget.size == FieldSize.large
                      ? cellTheme.cellLargeVerticalPadding
                      : cellTheme.cellVerticalPadding,
              horizontal: cellTheme.cellHorizontalPadding,
            ),
          ),
        ),
        type: MaterialType.transparency,
      ),
    );
  }

  Widget _builderCell() {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        Flex(
          direction: Axis.horizontal,
          mainAxisSize: MainAxisSize.max,
          crossAxisAlignment: this.widget.autoSize || this.widget.rows > 1
              ? CrossAxisAlignment.start
              : CrossAxisAlignment.center,
          children: <Widget>[
            this._buildLeft(),
            this._input(),
            this._right(),
          ],
        ),
        if (this.widget.errorMessage != null) this._errorMessage(),
      ],
    );
  }

  Widget _input() {
    final fieldTheme = VantTheme.of(context).field;

    return Flexible(
      child: Container(
          child: TextFormField(
        maxLines: this.widget.autoSize ? null : this.widget.rows ?? 1,
        minLines: this.widget.rows ?? null,
        maxLength: this.widget.maxLength,
        controller: this.widget.controller,
        focusNode: this._focusNode,
        keyboardType: this.widget.keyboardType,
        textInputAction: this.widget.textInputAction,
        textAlign: this.widget.inputAlign,
        autofocus: this.widget.autofocus,
        readOnly: this.widget.readonly,
        enabled: !this.widget.disabled,
        onChanged: (value) {
          if (this.widget.clearable) {
            if (value.isNotEmpty &&
                this._focusNode.hasFocus &&
                this.showClearable == false) {
              this.showClearable = true;
            } else if (this.showClearable == true) {
              this.showClearable = false;
            }

            setState(() {});
          }
        },
        style: TextStyle(
          color: this.widget.error
              ? fieldTheme.fieldInputErrorTextColor
              : fieldTheme.fieldInputTextColor,
        ),
        buildCounter: (BuildContext context,
            {int currentLength, int maxLength, bool isFocused}) {
          if (maxLength == null || !this.widget.showWordLimit) return null;
          return Text(
            "$currentLength/$maxLength",
            style: TextStyle(fontSize: 12.0, color: Colors.red),
          );
        },
        decoration: InputDecoration(
          isDense: true,
          contentPadding: EdgeInsets.zero,
          hintText: this.widget.placeholder,
          hintStyle: TextStyle(
            color: this.widget.error
                ? fieldTheme.fieldInputErrorTextColor
                : fieldTheme.fieldPlaceholderTextColor,
            fontSize: 14.0,
          ),
          border: InputBorder.none,
          errorStyle: TextStyle(
            fontSize: fieldTheme.fieldErrorMessageTextSize,
            color: fieldTheme.fieldRequiredColor,
          ),
        ),
      )),
      fit: FlexFit.tight,
    );
  }

  Widget _right() {
    final fieldTheme = VantTheme.of(context).field;

    if (this.widget.rightWidget != null) return this.widget.rightWidget;

    return Row(children: <Widget>[
      if (this.widget.clearable)
        Visibility(
          visible: this.showClearable,
          child: InkWell(
              child: Icon(
                CupertinoIcons.clear_circled_solid,
                color: fieldTheme.fieldClearIconColor,
                size: fieldTheme.fieldClearIconSize,
              ),
              onTap: () {
                this.widget.controller.clear();
              }),
        ),
      if (this.widget.rightIcon != null)
        Icon(
          this.widget.rightIcon,
          color: fieldTheme.fieldLabelColor,
          size: fieldTheme.fieldIconSize,
        )
    ]);
  }

  Widget _buildLeft() {
    final fieldTheme = VantTheme.of(context).field;

    return Row(children: <Widget>[
      if (this.widget.require)
        Text(
          "*",
          style: TextStyle(
            fontSize: 14.0,
            color: fieldTheme.fieldRequiredColor,
          ),
        ),
      if (this.widget.leftIcon != null)
        Padding(
          child: Icon(
            this.widget.leftIcon,
            color: fieldTheme.fieldLabelColor,
            size: fieldTheme.fieldIconSize,
          ),
          padding: EdgeInsets.only(top: 3, right: 2),
        ),
      if (this.widget.label != null)
        SizedBox(
          width: this.widget.label != null ? fieldTheme.fieldLabelWidth : 0,
          child: Text(
            this.widget.label,
            textAlign: this.widget.labelAlign,
            style: TextStyle(
                fontSize: this.widget.size == FieldSize.large
                    ? VantTheme.of(context).cell.cellLargeTitleFontSize
                    : VantTheme.of(context).cell.cellFontSize,
                color: fieldTheme.fieldLabelColor),
          ),
        )
    ]);
  }

  Widget _errorMessage() {
    final fieldTheme = VantTheme.of(context).field;

    return Container(
      alignment: this.widget.errorMessageAlign,
      child: Text(this.widget.errorMessage,
          style: TextStyle(
              color: fieldTheme.fieldErrorMessageColor,
              fontSize: fieldTheme.fieldErrorMessageTextSize)),
      margin: EdgeInsets.only(left: fieldTheme.fieldLabelWidth, top: 2),
      padding: EdgeInsets.only(left: 5),
    );
  }
}
