import 'package:flutter/material.dart';
import 'package:vant/vant.dart';

class VanTag extends StatefulWidget {
  final TagType type;
  final TagSize size;
  final Color color;
  final bool plain;
  final bool round;
  final bool mark;
  final Color textColor;
  final bool closeable;
  final Widget widgetText;
  final String text;

  VanTag({
    this.text,
    this.type,
    this.size,
    this.color,
    this.textColor,
    this.widgetText,
    this.plain = false,
    this.round = false,
    this.mark = false,
    this.closeable = false,
  });

  @override
  _VanTagState createState() => _VanTagState();
}

class _VanTagState extends State<VanTag> {
  bool visible = true;

  @override
  Widget build(BuildContext context) {
    final tagTheme = VantTheme.of(context).tag;

    return Visibility(
      visible: this.visible,
      child: Container(
        padding: VantTheme.of(context).tag.tagPadding,
        decoration: BoxDecoration(
          color: this.widget.color != null
              ? this.widget.color
              : this.widget.plain
                  ? tagTheme.tagPlainBackgroundColor
                  : tagTheme.color(this.widget.type),
          border: Border.all(
              width: VantTheme.of(context).basic.borderWidthBase,
              color: this.widget.color != null
                  ? this.widget.color
                  : tagTheme.color(this.widget.type)),
          borderRadius: this.widget.mark
              ? BorderRadius.horizontal(
                  left: Radius.circular(
                    tagTheme.tagBorderRadius,
                  ),
                  right: Radius.circular(tagTheme.tagRoundBorderRadius),
                )
              : BorderRadius.circular(widget.round
                  ? tagTheme.tagRoundBorderRadius
                  : tagTheme.tagBorderRadius),
        ),
        child: Row(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              widget.text,
              style: TextStyle(
                color: this.widget.textColor != null
                    ? this.widget.textColor
                    : this.widget.plain
                        ? this.widget.color ?? tagTheme.color(this.widget.type)
                        : tagTheme.tagTextColor,
                fontSize: tagTheme.size(this.widget.size),
              ),
            ),
            if (this.widget.closeable)
              Padding(
                  padding: EdgeInsets.only(left: 2.0),
                  child: GestureDetector(
                      child: Icon(
                        Icons.close,
                        color: tagTheme.tagTextColor,
                        size: tagTheme.size(this.widget.size),
                      ),
                      onTap: () {
                        setState(() {
                          this.visible = false;
                        });
                      }))
          ],
        ),
      ),
    );
  }
}
